  export default class {

  constructor(caseSensitive = false) {
    this.caseSensitive = caseSensitive;
    this.keyPressed = new Map();
    $(window).on("keydown", event => this.onKeyDown(event));
    $(window).on("keyup", event => this.onKeyUp(event));
    this.pubSub = $({});
  }

  onKey(key, callback) {
    if (!this.caseSensitive) {
      key = key.toUpperCase();
    }
    this.pubSub.on(`keyboard:${key}`, callback);
  }

  onKeyDown(event) {
    //if (event.originalEvent.repeat) return;
    let key = event.key;
    if (!this.caseSensitive) {
      key = key.toUpperCase();
    }
    this.keyPressed.set(key, key);
    this.pubSub.trigger(`keyboard:${key}`, this.keyPressed);
  }

  onKeyUp(event) {
    let key = event.key;
    if (!this.caseSensitive) {
      key = key.toUpperCase();
    }
    this.keyPressed.delete(key);
  }

  isKeyDown(key) {
    if (!this.caseSensitive) {
      key = key.toUpperCase();
    }
    return this.keyPressed.has(key);
  }

  isKeysDown(...keys) {
    return keys.every(key => this.isKeyDown(key));
  }

}
