import Bullet from "class/Circle/Bullet";

export default class {

  constructor(x, y , r, fireRate = 4, color = "red", turnSpeed = 0.003) {
    this.x = x;
    this.y = y;
    this.r = r;
    this.turnSpeed = turnSpeed;
    this.bullets = new Map();
    this.angle = 1.5 * Math.PI;
    this.gunX =this.x + 2 * this.r * Math.cos(this.angle);
    this.gunY = this.y + 2 * this.r * Math.sin(this.angle);
    this.bulletSize = 3;
    this.bulletSpeed = 0.5;
    this.fireRate = fireRate
    this.lastFireTime = -fireRate;
    this.color = color;
  }

  turn(dt, clockwise = true) {
    let turnRad = this.turnSpeed * dt;
    if (clockwise) {
        this.angle += turnRad;
    } else {
        this.angle -= turnRad;
    }
    this.angle = Math.min(this.angle, 2 * Math.PI);
    this.angle = Math.max(this.angle, Math.PI);
    this.gunX =this.x + 2 * this.r * Math.cos(this.angle);
    this.gunY = this.y + 2 * this.r * Math.sin(this.angle);
  }

  fire(elapsedTime) {
    if (elapsedTime - this.lastFireTime < 1000 / this.fireRate ) return;
    this.lastFireTime = elapsedTime;
    let c = new Bullet(this.gunX, this.gunY, this.bulletSize, this.bulletSpeed, this.angle);
    this.bullets.set(c, c);
  }

  fireBonus() {
    for (let radian = Math.PI; radian <= 2 * Math.PI; radian += 0.05){
      let c = new Bullet(this.x, this.y, this.bulletSize, this.bulletSpeed*1.5, radian);
      this.bullets.set(c, c);
    }
  }

  update(dt) {
    this.bullets.forEach(bullet => bullet.move(dt));
  }

  checkCollisions(targets) {
    this.bullets.forEach(bullet => {
      targets.forEach(target => {
        if (bullet.isCollidingWith(target)) {
          targets.delete(target);
          this.bullets.delete(bullet);
          if (target.isBonus()) this.fireBonus();
        }
      })
    });
  }

  draw(ctx) {
    ctx.beginPath();
    // draw the body
    ctx.fillStyle = this.color;
    ctx.strokeStyle = this.color;
    ctx.arc(this.x, this.y, this.r, Math.PI, 2*Math.PI);
    // draw the canon
    ctx.lineWidth = 5;
    ctx.moveTo(this.x, this.y);
    ctx.lineTo(this.gunX, this.gunY);
    // stroke and fill
    ctx.closePath();
    ctx.fill();
    ctx.stroke();
    // draw bullets
    this.bullets.forEach(bullet => bullet.draw(ctx));
  }

}