import mainloop from "MainLoop.js";
import Keyboard from "class/Keyboard";
import Shooter from "class/Shooter";
import Target from "class/Circle/Target";
import {getRandomInt} from "lib/Math";


const KEYBOARD = new Keyboard();
let ctx = document.querySelector("canvas").getContext("2d");
ctx.canvas.width = ctx.canvas.clientWidth;
ctx.canvas.height = ctx.canvas.clientHeight;

let shooter = new Shooter(ctx.canvas.width / 2, ctx.canvas.height, 30);
let score = 0;
let elapsedTime = 0;
let targets = new Map();

let generateTarget = () => {
    let c = new Target(getRandomInt(0, ctx.canvas.width), 0, 20, 0.1);
    targets.set(c, c);
}

let generateBonus = () => {
    let c = new Target(getRandomInt(0, ctx.canvas.width), 0, 40, 0.1, "pink", "red");
    targets.set(c, c);
}

setInterval(generateTarget, 1000);
setInterval(generateBonus, 15000);
generateTarget();
generateBonus();


let update = dt => {
    elapsedTime += dt;
    if (KEYBOARD.isKeyDown('a')) {
        shooter.turn(dt, false);
    }
    if (KEYBOARD.isKeyDown('d')) {
        shooter.turn(dt);
    }
    if (KEYBOARD.isKeyDown(' ')) {
        shooter.fire(elapsedTime);
    }
    shooter.update(dt);
    shooter.checkCollisions(targets);
    targets.forEach(target => target.move(Math.PI/2, dt));
    score = Math.round(elapsedTime / 100);
    targets.forEach(target => {
        if (target.y + target.r >= ctx.canvas.height) {
            mainloop.stop();
        }
    });
};
let drawScore = () => {
    ctx.font = "20px serif";
    ctx.fillStyle = "white";
    ctx.fillText(`Score: ${score}` , 60, 20);
}
let draw = dt => {
    ctx.canvas.width = ctx.canvas.clientWidth;
    ctx.canvas.height = ctx.canvas.clientHeight;
    targets.forEach(target => target.draw(ctx));
    shooter.draw(ctx);
    drawScore();
};

mainloop
    .setUpdate(update)
    .setDraw(draw)
    .start();